/**********************************************************************\
**
**                         Copyright (c) 2008-2022 by
**                          HENGSTLER GmbH, Aldingen 
**
**      This  software  is furnished under a license and may be used and
**      copied only in accordance with the terms  of  such  license  and
**      with the inclusion of the above copyright notice.  This software
**      or any other copies thereof may not  be  provided  or otherwise
**      made  available  to any other person.  No title to and ownership
**      of the software is hereby transferred.
**
**      The  information  in  this software is subject to change without
**      notice and should not be construed as a commitment by Hengstler.
**
**  Module:      Xpm_Pmpt_DlgMain.cpp
**  Facility:    S690001 - Hengstler XPM Printer Driver for Windows XP/7/8/10 
**  Component:   XPM print monitor plugin test tool (Xpm-PrintMonPluginTest)
**  Version:     R8-V1.07-FINAL / 2022OCT18
**  Environment: Windows XP/7/8/10
**  Compiler:    Microsoft Visual Studio 2015
**  Abstract:    Handler of main dialog
**  Note:        -
**  History:     R1-V1.00c / 2008JUL11 / HOR / 1st RELEASE - 1st BETA
**               R1-V1.00d / 2008DEC11 / HOR / 1st RELEASE - 2nd BETA
**               R1-V1.00e / 2009MAY05 / HOR / 1st RELEASE - 3rd BETA
**               R1-V1.00f / 2009JUL16 / HOR / 1st RELEASE - 4th BETA
**               R1-V1.00  / 2009AUG27 / HOR / 1st RELEASE - FINAL 
**               ------------------------------------------------------------
**               R2-V1.01a / 2010JAN26 / HOR / 2nd RELEASE - 1st BETA 
**               R2-V1.01b / 2010MAR04 / HOR / 2nd RELEASE - 2nd BETA
**               R2-V1.01c / 2011DEC19 / HOR / M-R2-012-HOR
**                  Complete driver package converted to VS2010 and adapted to Windows 7 32/64 bit
**               R2-V1.01c / 2011DEC19 / HOR / 2nd RELEASE - 3rd BETA
**               R2-V1.01d / 2012JUN27 / HOR / 2nd RELEASE - 4th BETA
**               R2-V1.01e / 2015MAY05 / HOR / 2nd RELEASE - 5th BETA
**               R2-V1.01f / 2016FEB15 / HOR / 2nd RELEASE - 6th BETA
**               R2-V1.01  / 2016AUG19 / HOR / M-R2-041-HOR
**                 Complete driver package converted to VS2015 and adapted to Windows 8/10 32/64 bit
**               R2-V1.01  / 2016AUG19 / HOR / 2nd RELEASE - FINAL
**               ------------------------------------------------------------
**               R3-V1.02  / 2016AUG24 / HOR / M-R3-001-HOR
**                 Additional driver package compilation added for usage within Windows-XP.
**               R3-V1.02  / 2016AUG24 / HOR / 3rd RELEASE - FINAL
**               ------------------------------------------------------------
**               R4-V1.03  / 2018JUL10 / HOR / 4th RELEASE - FINAL
**               ------------------------------------------------------------
**               R5-V1.04  / 2018OCT11 / HOR / 5th RELEASE - FINAL
**               ------------------------------------------------------------
**               R6-V1.05  / 2018DEC07 / HOR / 6th RELEASE - FINAL
**               ------------------------------------------------------------
**               R7-V1.06  / 2022FEB10 / HOR / 7th RELEASE - FINAL
**               ------------------------------------------------------------
**               R8-V1.07  / 2022OCT18 / HOR / 8th RELEASE - FINAL
**               ------------------------------------------------------------
**
\**********************************************************************/

/*
** INCLUDES
*/

#include "stdafx.h"
#include "winspool.h"
#include "..\..\Libraries\Xpm_PrintMonPlugin.h"
#include "..\..\Libraries\Xpm_Api.h"
#include "Xpm-Pmpt-App.h"
#include "Xpm-Pmpt-DlgMain.h"

/*
** COMPILER 
*/

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

/*
** MACROS
*/

/*
** TYPES
*/

/*
** FUNCTIONS
*/

/*
** VARIABLES
*/

/* 
** MESSAGE MAPS
*/

BEGIN_MESSAGE_MAP(CXpm_Pmpt_DlgMain, CDialog)
   ON_WM_SYSCOMMAND()
   ON_WM_PAINT()
   ON_WM_QUERYDRAGICON()
   //}}AFX_MSG_MAP
   ON_WM_TIMER()
   ON_CBN_SELCHANGE(IDC_COMBO_PRINTER_DRIVER, OnCbnSelchangeComboPrinterDriver)
END_MESSAGE_MAP()

//******************************************************************************************
//** METHOD:   CXpm_Pmpt_DlgMain::CXpm_Pmpt_DlgMain
//** ABSTRACT: Constructor
//******************************************************************************************

CXpm_Pmpt_DlgMain::CXpm_Pmpt_DlgMain
   (CWnd* pParent /*=NULL*/)
   : CDialog(CXpm_Pmpt_DlgMain::IDD, pParent)
{
   //-- Load dialog icons
   m_hIconL  =AfxGetApp()->LoadIcon(IDR_ICO_MAIN_L);
   m_hIconS = AfxGetApp()->LoadIcon(IDR_ICO_MAIN_S);

   //-- Initialize variables
   m_iTimer     = 0;
   m_pszPrinter = NULL;
}

//******************************************************************************************
//** METHOD:   CXpm_Pmpt_DlgMain::~CXpm_Pmpt_DlgMain
//** ABSTRACT: Destructor
//******************************************************************************************

CXpm_Pmpt_DlgMain::~CXpm_Pmpt_DlgMain
   ()
{
   //-- Free printer if allocated
   if (m_pszPrinter) free(m_pszPrinter); m_pszPrinter= NULL;
}

//******************************************************************************************
//** METHOD:   CXpm_Pmpt_DlgMain::DoDataExchange
//** ABSTRACT: This method is called by the framework to exchange and validate dialog data.
//******************************************************************************************

void CXpm_Pmpt_DlgMain::DoDataExchange
   (CDataExchange* pDX)
{
   //-- ...
   CDialog::DoDataExchange(pDX);
}

//******************************************************************************************
//** METHOD:   CXpm_Pmpt_DlgMain::OnInitDialog
//** ABSTRACT: This method is called by the framework in response to the WM_INITDIALOG message
//******************************************************************************************

BOOL CXpm_Pmpt_DlgMain::OnInitDialog
   ()
{
   //-- Init dialog
   CDialog::OnInitDialog();

   //-- Set dialog icons
   SetIcon(m_hIconL, TRUE);   
   SetIcon(m_hIconS, FALSE);  

   //-- Modify system menu and add dialogs
   CMenu* pSysMenu = this->GetSystemMenu(FALSE);

   //-- Setup combo box: printer driver
   { 
      CComboBox * poComboBox;   // Pointer to combo box
 
      //-- Get pointer to combo box
      if ((poComboBox = ((CComboBox *) this->GetDlgItem(IDC_COMBO_PRINTER_DRIVER))) != NULL) {

         HANDLE hPrinterEnum;  // Handle of printer enumeration

         //-- Reset contents of box
         poComboBox->ResetContent() ;

         //-- Start printer enumeration
         if ((hPrinterEnum = Xpm_Api_Pe_PrinterEnumStart()) != NULL) {

            PSTR pszPrinterName;  // Name of printer

            //-- Go through all printers
            for (pszPrinterName = Xpm_Api_Pe_PrinterEnum(hPrinterEnum, true); pszPrinterName; pszPrinterName = Xpm_Api_Pe_PrinterEnum(hPrinterEnum, false)) {
 
                //-- Add printer
                poComboBox->AddString(pszPrinterName);
            }

            //-- End of printer enumeration
            Xpm_Api_Pe_PrinterEnumEnd(hPrinterEnum);
         }
 
         //-- Select printer
         poComboBox->SetCurSel(0);
      }
   }
   
   //-- Get current driver name 
   OnCbnSelchangeComboPrinterDriver();

   //-- Update dialog
   UpdateDialog();

   //-- Setup update timer to .. hz
   m_iTimer = SetTimer(1, 1000, NULL);

   //-- All done
   return TRUE; 
}

//******************************************************************************************
//** METHOD:   CXpm_Pmpt_DlgMain::OnSysCommand
//** ABSTRACT: This method is called by the framework when the user selects a command from 
//**          the Control menu, or when the user selects the Maximize or the Minimize button.
//******************************************************************************************

void CXpm_Pmpt_DlgMain::OnSysCommand
   (UINT nID, LPARAM lParam)
{
    CDialog::OnSysCommand(nID, lParam);
}

//******************************************************************************************
//** METHOD:   CXpm_Pmpt_DlgMain::OnPaint
//** ABSTRACT: This method is called by the framework when an application makes a request to 
//**          repaint a portion of an application window. 
//******************************************************************************************

void CXpm_Pmpt_DlgMain::OnPaint
   () 
{
   if (IsIconic()) {

      CPaintDC dc(this); //--Gertekontext zum Zeichnen

      SendMessage(WM_ICONERASEBKGND, reinterpret_cast<WPARAM>(dc.GetSafeHdc()), 0);

      //-- Symbol in Clientrechteck zentrieren
      int cxIcon = GetSystemMetrics(SM_CXICON);
      int cyIcon = GetSystemMetrics(SM_CYICON);
      CRect rect;
      GetClientRect(&rect);
      int x = (rect.Width() - cxIcon + 1) / 2;
      int y = (rect.Height() - cyIcon + 1) / 2;

      //-- Symbol zeichnen
      dc.DrawIcon(x, y, this->m_hIconS);
   }
   else {
      CDialog::OnPaint();
   }
}

//******************************************************************************************
//** METHOD:   CXpm_Pmpt_DlgMain::OnQueryDragIcon
//** ABSTRACT: The framework calls this member function by a minimized (iconic) window that 
//**          does not have an icon defined for its class
//******************************************************************************************

HCURSOR CXpm_Pmpt_DlgMain::OnQueryDragIcon
   ()
{
   return static_cast<HCURSOR>(this->m_hIconS);
}

//******************************************************************************************
//** METHOD     CXpm_Pmpt_DlgMain::OnTimer
//** ABSTRACT   Cyclic timer event (1 sec ) to update structures from registry and 
//             dialog elements
//******************************************************************************************

void CXpm_Pmpt_DlgMain::OnTimer
   (UINT nIDEvent)
{
   //-- Update dialog
   UpdateDialog();
}

//******************************************************************************************
//** METHOD     CXpm_Pmpt_DlgMain::OnCbnSelchangeComboPrinterDriver
//** ABSTRACT   Handles printer driver selection combo box
//******************************************************************************************

void CXpm_Pmpt_DlgMain::OnCbnSelchangeComboPrinterDriver
   ()
{
   //-- Free printer if allocated
   if (m_pszPrinter) free(m_pszPrinter); m_pszPrinter= NULL;

   //-- Get combo box selection: printer driver
   { 
      CComboBox * poComboBox;   // Pointer to combo box
      INT         iSel;         // Selected item
 
      //-- Get pointer to combo box
      if ((poComboBox = ((CComboBox *) this->GetDlgItem(IDC_COMBO_PRINTER_DRIVER))) != NULL) {

         //-- Get selected item
         if ((iSel = poComboBox->GetCurSel()) != CB_ERR) {

            //-- Allocate memory for printer name 
            if ((m_pszPrinter = (PSTR) malloc(poComboBox->GetLBTextLen(iSel)+1)) != NULL) {

               //-- Get text of selected item
               poComboBox->GetLBText(iSel, m_pszPrinter);
            }
         }
      }
   }

   //-- Update dialog
   UpdateDialog();
}

//******************************************************************************************
//** METHOD     CXpm_Pmpt_DlgMain::ReadPrinterDataFromRegistry
//** ABSTRACT   Reads printer data from registry
//******************************************************************************************

bool CXpm_Pmpt_DlgMain::ReadPrinterDataFromRegistry
   (PSTR pszPrinter, PSTR pszKey, PSTR pszValue, PBYTE pubData, DWORD dwSize)
{
   bool                            fResult                    = false;  // Return value, FALSE by default
   PRINTER_DEFAULTS                sPrinterDefaults;                    // Printer defaults structure
   HANDLE                          hPrinter                   = NULL;   // Handle of printer

   //-- Validate parameter
   if (!pszPrinter || !pszKey || !pszValue || !pubData || !dwSize)
      goto FAILURE;

   //-- Setup printer defaults
   ZeroMemory(&sPrinterDefaults, sizeof(sPrinterDefaults));
   sPrinterDefaults.DesiredAccess = PRINTER_ALL_ACCESS;
  
   //-- Open printer 
   if (!OpenPrinter(pszPrinter, &hPrinter, &sPrinterDefaults)) 
      goto FAILURE;

   //-- Read data from registery and reset data if not (yet) available or if revision nr differs
   {
      DWORD dwType;   //--Type of registry data
      DWORD dwNeeded; //--Bytes needed
   
       if (    (GetPrinterDataExA(hPrinter, pszKey, pszValue, &dwType, pubData, dwSize, &dwNeeded)  != ERROR_SUCCESS) 
            || (dwType != REG_BINARY)
          ) {
            goto FAILURE;
      }
   }

   //-- Close printer
   ClosePrinter(hPrinter); hPrinter = NULL;

   //-- Success
   fResult = true;

   //-- All done
   goto EXIT;

FAILURE:
   ;

EXIT:

   //-- Close printer if open
   if (hPrinter) ClosePrinter(hPrinter); hPrinter = NULL;

   //-- Return TRUE on success and FALSE otherwise
   return(fResult);
}

//************************************************************************
//** METHOD   CXpm_Pmpt_DlgMain::UpdateJobInfoInDialog
//** ABSTRACT Updates job information in the main dialog
//************************************************************************

void CXpm_Pmpt_DlgMain::UpdateJobInfoInDialog
   (PS_XPM_PMP_JOB_INFO psJobInfo)
{
   DWORD                dwAgeSec   = -1;        // Age of job information in sec
   CString              strItem;                // A string

   //-- Update registry path to job information
   {
      //-- Path is ...\KEY\VALUE
      strItem += "...\\";
      strItem += D_XPM_PMP_REG_KEY;
      strItem += "\\";
      strItem += D_XPM_PMP_REG_VALUE_PMP_JOB_INFO;
      GetDlgItem(IDC_TXT_JI_REGISTRY_ENTRY)->SetWindowText(strItem);
   }

   //-- Determine age of job information if valid
   if (psJobInfo && (psJobInfo->dwTsLastUpdate != (DWORD) -1)) 
        dwAgeSec = (GetTickCount() - psJobInfo->dwTsLastUpdate) / 1000;
   else dwAgeSec = (DWORD) -1; 

   //-- Update age of job information
   {
      //-- Regard update timestamp valid if ...
      //..     job info is valid 
      //.. AND age differs from 0xFFFFFFFF
      if (psJobInfo && (dwAgeSec != (DWORD) -1)) 
           strItem.Format("%u sec", dwAgeSec);
      else strItem = "<INVALID>";
      GetDlgItem(IDC_TXT_JI_TIME_SINCE_LAST_UPDATE)->SetWindowText(strItem);
   }

   //-- Update job identifier
   { 
      //-- Regard job identifier valid if ...
      //..     job info is valid 
      //.. AND age less than ... ms 
      if (psJobInfo && (dwAgeSec < 99999)) {
         if (psJobInfo->dwJobId != (DWORD) -1)
              strItem.Format("%d", psJobInfo->dwJobId);
         else strItem.Format("<UNDEFINED>");
      }
      else strItem = "<INVALID>";
      GetDlgItem(IDC_TXT_JI_JOB_IDENTIFIER)->SetWindowText(strItem);
   }

   //-- Update job status
   { 
      //-- Regard job status valid if ...
      //..     job info is valid 
      //.. AND age less than ... ms 
      if (psJobInfo && (dwAgeSec < 99999)) {
         if (psJobInfo->dwJobId != (DWORD) -1)
              strItem.Format("%d", psJobInfo->dwJobStatus);
         else strItem.Format("<UNDEFINED>");
      }
      else strItem = "<INVALID>";
      GetDlgItem(IDC_TXT_JI_JOB_STATUS)->SetWindowText(strItem);
   }

   //-- Update page count 
   {
      //-- Regard page count valid if ...
      //..     job info is valid 
      //.. AND age less than ... ms 
      if (psJobInfo && (dwAgeSec < 99999)) {
         if (psJobInfo->dwJobId != (DWORD) -1)
              strItem.Format("%d", psJobInfo->dwPageCount);
         else strItem.Format("<UNDEFINED>");
      }
      else strItem = "<INVALID>";
      GetDlgItem(IDC_TXT_JI_PAGE_COUNT)->SetWindowText(strItem);
   }
}

//************************************************************************
//** METHOD   CXpm_Pmpt_DlgMain::UpdatePrinterInfoInDialog
//** ABSTRACT Updates printer information in the main dialog
//************************************************************************

void CXpm_Pmpt_DlgMain::UpdatePrinterInfoInDialog
   (PS_XPM_PMP_PRINTER_INFO psPrinterInfo)
{
   DWORD                dwAgeSec   = -1;        // Age of printer information in seconds
   CString              strItem;                // A string

   //-- Update registry path to printer information structure
   {
      //-- Path is ...\KEY\VALUE
      strItem += "...\\";
      strItem += D_XPM_PMP_REG_KEY;
      strItem += "\\";
      strItem += D_XPM_PMP_REG_VALUE_PMP_PRINTER_INFO;
      GetDlgItem(IDC_TXT_PIN_REGISTRY_ENTRY)->SetWindowText(strItem);
   }

   //-- Determine age of printer information if valid 
   if (psPrinterInfo && (psPrinterInfo->dwTsLastUpdate != (DWORD) -1)) 
        dwAgeSec = (GetTickCount() - psPrinterInfo->dwTsLastUpdate) / 1000;
   else dwAgeSec = (DWORD) -1; 

   //-- Update age of printer information
   {
      //-- Regard update timestamp valid if ...
      //..     printer info is valid 
      //.. AND age differs from 0xFFFFFFFF
      if (psPrinterInfo && (dwAgeSec != (DWORD) -1))
           strItem.Format("%u sec", dwAgeSec);
      else strItem = "<INVALID>";
      GetDlgItem(IDC_TXT_PIN_TIME_SINCE_LAST_UPDATE)->SetWindowText(strItem);
   }

   //-- Update printer part number 
   { 
      //-- Regard printer part number valid if ...
      //..     printer info is valid 
      //.. AND age less than ... ms 
      if (psPrinterInfo && (dwAgeSec < 99999)) {
         if (    (psPrinterInfo->dwDataValidFlag & D_XPM_PMP_PIN_DVF_PRINTER_INFO) 
              && psPrinterInfo->sPrinterInfo.szPartNr[0]
            ) strItem = psPrinterInfo->sPrinterInfo.szPartNr;
         else strItem.Format("<UNDEFINED>");
      }
      else strItem = "<INVALID>";
      GetDlgItem(IDC_TXT_PIN_PART_NUMBER)->SetWindowText(strItem);
   }

   //-- Update printer serial number 
   { 
      //-- Regard printer serial number valid if ...
      //..     printer info is valid 
      //.. AND age less than ... ms 
      if (psPrinterInfo && (dwAgeSec < 99999)) {
         if (    (psPrinterInfo->dwDataValidFlag & D_XPM_PMP_PIN_DVF_PRINTER_INFO) 
              && psPrinterInfo->sPrinterInfo.szSerialNr[0]
            ) strItem = psPrinterInfo->sPrinterInfo.szSerialNr;
         else strItem.Format("<UNDEFINED>");
      }
      else strItem = "<INVALID>";
      GetDlgItem(IDC_TXT_PIN_SERIAL_NUMBER)->SetWindowText(strItem);
   }

   //-- Update date of manufacturing 
   { 
      //-- Regard date of manufacturing valid if ...
      //..     printer info is valid 
      //.. AND age less than ... ms 
      if (psPrinterInfo && (dwAgeSec < 99999)) {
         if (    (psPrinterInfo->dwDataValidFlag & D_XPM_PMP_PIN_DVF_PRINTER_INFO) 
              && psPrinterInfo->sPrinterInfo.szDateOfManuf[0]
            ) strItem = psPrinterInfo->sPrinterInfo.szDateOfManuf;
         else strItem.Format("<UNDEFINED>");
      }
      else strItem = "<INVALID>";
      GetDlgItem(IDC_TXT_PIN_DATE_OF_MANUFACTURING)->SetWindowText(strItem);
   }

   //-- Update model code
   { 
      //-- Regard model code valid if ...
      //..     printer info is valid 
      //.. AND age less than ... ms 
      if (psPrinterInfo && (dwAgeSec < 99999)) {
         if (psPrinterInfo->dwDataValidFlag & D_XPM_PMP_PIN_DVF_PRINTER_INFO) 
              strItem.Format("[%8.8x]", psPrinterInfo->sPrinterInfo.dwModelCode);
         else strItem.Format("<UNDEFINED>");
      }
      else strItem = "<INVALID>";
      GetDlgItem(IDC_TXT_PIN_MODEL_CODE)->SetWindowText(strItem);
   }

   //-- Update configuration code
   { 
      //-- Regard configuration code valid if ...
      //..     printer info is valid 
      //.. AND age less than ... ms 
      if (psPrinterInfo && (dwAgeSec < 99999)) {
         if (psPrinterInfo->dwDataValidFlag & D_XPM_PMP_PIN_DVF_PRINTER_INFO) 
              strItem.Format("[%8.8x]", psPrinterInfo->sPrinterInfo.dwConfigCode);
         else strItem.Format("<UNDEFINED>");
      }
      else strItem = "<INVALID>";
      GetDlgItem(IDC_TXT_PIN_CONFIGURATION_CODE)->SetWindowText(strItem);
   }

   //-- Update sensor code
   { 
      //-- Regard sensor code valid if ...
      //..     printer info is valid 
      //.. AND age less than ... ms 
      if (psPrinterInfo && (dwAgeSec < 99999)) {
         if (psPrinterInfo->dwDataValidFlag & D_XPM_PMP_PIN_DVF_PRINTER_INFO) 
              strItem.Format("[%8.8x]", psPrinterInfo->sPrinterInfo.dwSensorCode);
         else strItem.Format("<UNDEFINED>");
      }
      else strItem = "<INVALID>";
      GetDlgItem(IDC_TXT_PIN_SENSOR_CODE)->SetWindowText(strItem);
   }

   //-- Update feature code
   { 
      //-- Regard feature code valid if ...
      //..     printer info is valid 
      //.. AND age less than ... ms 
      if (psPrinterInfo && (dwAgeSec < 99999)) {
         if (psPrinterInfo->dwDataValidFlag & D_XPM_PMP_PIN_DVF_PRINTER_INFO) 
              strItem.Format("[%8.8x]", psPrinterInfo->sPrinterInfo.dwFeatureCode);
         else strItem.Format("<UNDEFINED>");
      }
      else strItem = "<INVALID>";
      GetDlgItem(IDC_TXT_PIN_FEATURE_CODE)->SetWindowText(strItem);
   }

   //-- Update firmware part number
   { 
      //-- Regard firmware part number valid if ...
      //..     printer info is valid 
      //.. AND age less than ... ms 
      if (psPrinterInfo && (dwAgeSec < 99999)) {
         if (psPrinterInfo->dwDataValidFlag & D_XPM_PMP_PIN_DVF_PRINTER_INFO) 
              strItem = psPrinterInfo->sPrinterInfo.szFwPartNr;
         else strItem.Format("<UNDEFINED>");
      }
      else strItem = "<INVALID>";
      GetDlgItem(IDC_TXT_PIN_FIRMWARE_PART_NR)->SetWindowText(strItem);
   }

   //-- Update firmware version
   { 
      //-- Regard firmware version valid if ...
      //..     printer info is valid 
      //.. AND age less than ... ms 
      if (psPrinterInfo && (dwAgeSec < 99999)) {
         if (psPrinterInfo->dwDataValidFlag & D_XPM_PMP_PIN_DVF_PRINTER_INFO) 
              strItem = psPrinterInfo->sPrinterInfo.szFwVersion;
         else strItem.Format("<UNDEFINED>");
      }
      else strItem = "<INVALID>";
      GetDlgItem(IDC_TXT_PIN_FIRMWARE_VERSION)->SetWindowText(strItem);
   }

   //-- Update firmware date
   { 
      //-- Regard firmware date valid if ...
      //..     printer info is valid 
      //.. AND age less than ... ms 
      if (psPrinterInfo && (dwAgeSec < 99999)) {
         if (psPrinterInfo->dwDataValidFlag & D_XPM_PMP_PIN_DVF_PRINTER_INFO) 
              strItem = psPrinterInfo->sPrinterInfo.szFwDate;
         else strItem.Format("<UNDEFINED>");
      }
      else strItem = "<INVALID>";
      GetDlgItem(IDC_TXT_PIN_FIRMWARE_DATE)->SetWindowText(strItem);
   }
}

//************************************************************************
//** METHOD   CXpm_Pmpt_DlgMain::UpdatePrinterStatusInDialog
//** ABSTRACT Updates status information in the main dialog
//************************************************************************

void CXpm_Pmpt_DlgMain::UpdatePrinterStatusInDialog
   (PS_XPM_PMP_PRINTER_STATUS psPrinterStatus)
{
   DWORD                dwAgeSec   = -1;        // Age of status information in seconds
   CString              strItem;                // A string

   //-- Update registry path to printer status structure
   {
      //-- Path is ...\KEY\VALUE
      strItem += "...\\";
      strItem += D_XPM_PMP_REG_KEY;
      strItem += "\\";
      strItem += D_XPM_PMP_REG_VALUE_PMP_PRINTER_STATUS;
      GetDlgItem(IDC_TXT_PST_REGISTRY_ENTRY)->SetWindowText(strItem);
   }

   //-- Determine age of status information if valid 
   if (psPrinterStatus && (psPrinterStatus->dwTsLastUpdate != (DWORD) -1)) 
        dwAgeSec = (GetTickCount() - psPrinterStatus->dwTsLastUpdate) / 1000;
   else dwAgeSec = (DWORD) -1; 

   //-- Update age of printer information
   {
      //-- Regard update timestamp valid if ...
      //..     printer info is valid 
      //.. AND age differs from 0xFFFFFFFF
      if (psPrinterStatus && (dwAgeSec != (DWORD) -1))
           strItem.Format("%u sec", dwAgeSec);
      else strItem = "<INVALID>";
      GetDlgItem(IDC_TXT_PST_TIME_SINCE_LAST_UPDATE)->SetWindowText(strItem);
   }

   //-- Update status bits
   { 
      //-- Regard status bits valid if ...
      //..     printer status is valid 
      //.. AND age less than ... ms 
      if (psPrinterStatus && (dwAgeSec < 99999)) {
         if (psPrinterStatus->dwDataValidFlag & D_XPM_PMP_PST_DVF_PRINTER_STATUS) 
              strItem.Format("[%8.8x]", psPrinterStatus->sPrinterStatus.dwStatus);
         else strItem.Format("<UNDEFINED>");
      } 
      else strItem = "<INVALID>";
      GetDlgItem(IDC_TXT_PST_STATUS_BITS)->SetWindowText(strItem);
   }

   //-- Update job identifier
   { 
      //-- Regard job identifier valid if ...
      //..     printer status is valid  
      //.. AND age less than ... ms 
      if (psPrinterStatus && (dwAgeSec < 99999)) {
         if (psPrinterStatus->dwDataValidFlag & D_XPM_PMP_PST_DVF_PRINTER_STATUS) 
              strItem.Format("%d", (WORD) psPrinterStatus->sPrinterStatus.ubPrintJobId);
         else strItem.Format("<UNDEFINED>");
      }
      else strItem = "<INVALID>";
      GetDlgItem(IDC_TXT_PST_JOB_IDENTIFIER)->SetWindowText(strItem);
   }

   //-- Update print head temperature
   { 
      //-- Regard print head temperature valid if ...
      //..     printer status is valid  
      //.. AND age less than ... ms 
      if (psPrinterStatus && (dwAgeSec < 99999)) {
         if (psPrinterStatus->dwDataValidFlag & D_XPM_PMP_PST_DVF_PRINTER_STATUS) 
              strItem.Format("%+d C", (INT) psPrinterStatus->sPrinterStatus.siPrintHeadTempC);
         else strItem.Format("<UNDEFINED>");
      }
      else strItem = "<INVALID>";
      GetDlgItem(IDC_TXT_PST_PH_TEMPERATURE)->SetWindowText(strItem);
   }

   //-- Update board voltage
   { 
      //-- Regard board voltage valid if ...
      //..     printer status is valid  
      //.. AND age less than ... ms 
      if (psPrinterStatus && (dwAgeSec < 99999)) {
         if (psPrinterStatus->dwDataValidFlag & D_XPM_PMP_PST_DVF_PRINTER_STATUS) 
              strItem.Format("%2.1lf V", (double) psPrinterStatus->sPrinterStatus.wBoardVoltageTv / 10);
         else strItem.Format("<UNDEFINED>");
      }
      else strItem = "<INVALID>";
      GetDlgItem(IDC_TXT_PST_BOARD_VOLTAGE)->SetWindowText(strItem);
   }

   //-- Update printer state
   { 
      //-- Regard printer state valid if ...
      //..     printer status is valid 
      //.. AND age less than ... ms 
      if (psPrinterStatus && (dwAgeSec < 99999)) {
         if (psPrinterStatus->dwDataValidFlag & D_XPM_PMP_PST_DVF_PRINTER_STATUS) 
              strItem.Format("%d", (WORD) psPrinterStatus->sPrinterStatus.ubState);
         else strItem.Format("<UNDEFINED>");
      }
      else strItem = "<INVALID>";
      GetDlgItem(IDC_TXT_PST_PRINTER_STATE)->SetWindowText(strItem);
   }

   //-- Update printer error
   { 
      //-- Regard printer error if ...
      //..     printer status is valid 
      //.. AND age less than ... ms 
      if (psPrinterStatus && (dwAgeSec < 99999)) {
         if (psPrinterStatus->dwDataValidFlag & D_XPM_PMP_PST_DVF_PRINTER_STATUS) 
              strItem.Format("%d", (WORD) psPrinterStatus->sPrinterStatus.wErrorCode);
         else strItem.Format("<UNDEFINED>");
      }
      else strItem = "<INVALID>";
      GetDlgItem(IDC_TXT_PST_PRINTER_ERROR)->SetWindowText(strItem);
   }
}

//************************************************************************
//** METHOD   CXpm_Pmpt_DlgMain::UpdatePrinterPresentInDialog
//** ABSTRACT Updates printer presenter status information in the main dialog
//************************************************************************

void CXpm_Pmpt_DlgMain::UpdatePrinterPresentInDialog
   (PS_XPM_PMP_PRINTER_PRESENT psPrinterPresent)
{
   DWORD                dwAgeSec   = -1;        // Age of printer present in seconds
   CString              strItem;                // A string

   //-- Update registry path to printer present structure
   {
      //-- Path is ...\KEY\VALUE
      strItem += "...\\";
      strItem += D_XPM_PMP_REG_KEY;
      strItem += "\\";
      strItem += D_XPM_PMP_REG_VALUE_PMP_PRINTER_PRESENT;
      GetDlgItem(IDC_TXT_PPR_REGISTRY_ENTRY)->SetWindowText(strItem);
   }

   //-- Determine age of printer present data if valid 
   if (psPrinterPresent && (psPrinterPresent->dwTsLastUpdate != (DWORD) -1)) 
        dwAgeSec = (GetTickCount() - psPrinterPresent->dwTsLastUpdate) / 1000;
   else dwAgeSec = (DWORD) -1; 

   //-- Update age of printer information
   {
      //-- Regard update timestamp valid if ...
      //..     printer info is valid 
      //.. AND age differs from 0xFFFFFFFF
      if (psPrinterPresent && (dwAgeSec != (DWORD) -1))
           strItem.Format("%u sec", dwAgeSec);
      else strItem = "<INVALID>";
      GetDlgItem(IDC_TXT_PPR_TIME_SINCE_LAST_UPDATE)->SetWindowText(strItem);
   }

   //-- Update presenter type 
   { 
      //-- Regard presenter type valid if ...
      //..     printer presenter is valid  
      //.. AND age less than ... ms 
      if (psPrinterPresent && (dwAgeSec < 99999)) {
         if (psPrinterPresent->dwDataValidFlag & D_XPM_PMP_PPR_DVF_PRINTER_PRESENT) 
              strItem.Format("%d", (WORD) psPrinterPresent->sPrinterPresent.ubType);
         else strItem.Format("<UNDEFINED>");
      }
      else strItem = "<INVALID>";
      GetDlgItem(IDC_TXT_PPR_PRESENTER_TYPE)->SetWindowText(strItem);
   }


   //-- Update status bits
   { 
      //-- Regard presenter status bits valid if ...
      //..     printer presenter is valid 
      //.. AND age less than ... ms 
      if (psPrinterPresent && (dwAgeSec < 99999)) {
         if (psPrinterPresent->dwDataValidFlag & D_XPM_PMP_PPR_DVF_PRINTER_PRESENT) 
              strItem.Format("[%4.4x]", psPrinterPresent->sPrinterPresent.wStatus);
         else strItem.Format("<UNDEFINED>");
      } 
      else strItem = "<INVALID>";
      GetDlgItem(IDC_TXT_PPR_PRESENTER_STATUS)->SetWindowText(strItem);
   }

   //-- Update presenter state
   { 
      //-- Regard presenter state valid if ...
      //..     printer presenter is valid 
      //.. AND age less than ... ms 
      if (psPrinterPresent && (dwAgeSec < 99999)) {
         if (psPrinterPresent->dwDataValidFlag & D_XPM_PMP_PPR_DVF_PRINTER_PRESENT) 
              strItem.Format("%d", (WORD) psPrinterPresent->sPrinterPresent.ubState);
         else strItem.Format("<UNDEFINED>");
      }
      else strItem = "<INVALID>";
      GetDlgItem(IDC_TXT_PPR_PRESENTER_STATE)->SetWindowText(strItem);
   }

   //-- Update number of loaded printouts 
   { 
      //-- Regard number of loaded printouts valid if ...
      //..     printer presenter is valid  
      //.. AND age less than ... ms 
      if (psPrinterPresent && (dwAgeSec < 99999)) {
         if (psPrinterPresent->dwDataValidFlag & D_XPM_PMP_PPR_DVF_PRINTER_PRESENT) 
              strItem.Format("%d", (INT) psPrinterPresent->sPrinterPresent.ubNrOfLoadedPrintouts);
         else strItem.Format("<UNDEFINED>");
      }
      else strItem = "<INVALID>";
      GetDlgItem(IDC_TXT_PPR_PRESENTER_NUMBER_OF_LOADED_PRINTOUTS)->SetWindowText(strItem);
   }

   //-- Update total number of loaded printouts 
   { 
      //-- Regard total number of loaded printouts valid if ...
      //..     printer presenter is valid  
      //.. AND age less than ... ms 
      if (psPrinterPresent && (dwAgeSec < 99999)) {
         if (psPrinterPresent->dwDataValidFlag & D_XPM_PMP_PPR_DVF_PRINTER_PRESENT) 
              strItem.Format("%d", psPrinterPresent->sPrinterPresent.dwNrOfLoadedPrintouts);
         else strItem.Format("<UNDEFINED>");
      }
      else strItem = "<INVALID>";
      GetDlgItem(IDC_TXT_PPR_PRESENTER_TOTAL_NUMBER_OF_LOADED_PRINTOUTS)->SetWindowText(strItem);
   }

   //-- Update total number of taken printouts 
   { 
      //-- Regard total number of taken printouts valid if ...
      //..     printer presenter is valid  
      //.. AND age less than ... ms 
      if (psPrinterPresent && (dwAgeSec < 99999)) {
         if (psPrinterPresent->dwDataValidFlag & D_XPM_PMP_PPR_DVF_PRINTER_PRESENT) 
              strItem.Format("%d", psPrinterPresent->sPrinterPresent.dwNrOfTakenPrintouts);
         else strItem.Format("<UNDEFINED>");
      }
      else strItem = "<INVALID>";
      GetDlgItem(IDC_TXT_PPR_PRESENTER_TOTAL_NUMBER_OF_TAKEN_PRINTOUTS)->SetWindowText(strItem);
   }

   //-- Update total number of rejected printouts 
   { 
      //-- Regard total number of rejected printouts valid if ...
      //..     printer presenter is valid  
      //.. AND age less than ... ms 
      if (psPrinterPresent && (dwAgeSec < 99999)) {
         if (psPrinterPresent->dwDataValidFlag & D_XPM_PMP_PPR_DVF_PRINTER_PRESENT) 
              strItem.Format("%d", psPrinterPresent->sPrinterPresent.dwNrOfRejectedPrintouts);
         else strItem.Format("<UNDEFINED>");
      }
      else strItem = "<INVALID>";
      GetDlgItem(IDC_TXT_PPR_PRESENTER_TOTAL_NUMBER_OF_REJECTED_PRINTOUTS)->SetWindowText(strItem);
   }
}

//************************************************************************
//** METHOD   CXpm_Pmpt_DlgMain::UpdatePrinterFeedInDialog
//** ABSTRACT Updates printer feed status information in the main dialog
//************************************************************************

void CXpm_Pmpt_DlgMain::UpdatePrinterFeedInDialog
   (PS_XPM_PMP_PRINTER_FEED psPrinterFeed)
{
   DWORD                dwAgeSec   = -1;        // Age of printer present in seconds
   CString              strItem;                // A string

   //-- Update registry path to printer feed structure
   {
      //-- Path is ...\KEY\VALUE
      strItem += "...\\";
      strItem += D_XPM_PMP_REG_KEY;
      strItem += "\\";
      strItem += D_XPM_PMP_REG_VALUE_PMP_PRINTER_FEED;
      GetDlgItem(IDC_TXT_PFE_REGISTRY_ENTRY)->SetWindowText(strItem);
   }

   //-- Determine age of printer feed data if valid 
   if (psPrinterFeed && (psPrinterFeed->dwTsLastUpdate != (DWORD) -1)) 
        dwAgeSec = (GetTickCount() - psPrinterFeed->dwTsLastUpdate) / 1000;
   else dwAgeSec = (DWORD) -1; 

   //-- Update age of printer information
   {
      //-- Regard update timestamp valid if ...
      //..     printer info is valid 
      //.. AND age differs from 0xFFFFFFFF
      if (psPrinterFeed && (dwAgeSec != (DWORD) -1))
           strItem.Format("%u sec", dwAgeSec);
      else strItem = "<INVALID>";
      GetDlgItem(IDC_TXT_PFE_TIME_SINCE_LAST_UPDATE)->SetWindowText(strItem);
   }

   //-- Update feeder state
   { 
      //-- Regard feeder state valid if ...
      //..     printer feeder is valid 
      //.. AND age less than ... ms 
      if (psPrinterFeed && (dwAgeSec < 99999)) {
         if (psPrinterFeed->dwDataValidFlag & D_XPM_PMP_PFE_DVF_PRINTER_FEED) 
              strItem.Format("%d", (WORD) psPrinterFeed->sPrinterFeed.ubState);
         else strItem.Format("<UNDEFINED>");
      }
      else strItem = "<INVALID>";
      GetDlgItem(IDC_TXT_PFE_FEED_UNIT_STATE)->SetWindowText(strItem);
   }

   //-- Update number of paper paths 
   { 
      //-- Regard number of paper paths valid if ...
      //..     printer feeder is valid  
      //.. AND age less than ... ms 
      if (psPrinterFeed && (dwAgeSec < 99999)) {
         if (psPrinterFeed->dwDataValidFlag & D_XPM_PMP_PFE_DVF_PRINTER_FEED) 
              strItem.Format("%d", (INT) psPrinterFeed->sPrinterFeed.ubNrOfPaperPaths);
         else strItem.Format("<UNDEFINED>");
      }
      else strItem = "<INVALID>";
      GetDlgItem(IDC_TXT_PFE_NUMBER_OF_PAPER_PATHS)->SetWindowText(strItem);
   }

   //-- Update paper path status 
   { 
      //-- Regard paper path status valid if ...
      //..     printer feeder is valid 
      //.. AND age less than ... ms 
      if (psPrinterFeed && (dwAgeSec < 99999)) {
         if (psPrinterFeed->dwDataValidFlag & D_XPM_PMP_PFE_DVF_PRINTER_FEED) {
            strItem.Format("");
            for (int iPath = 0; iPath < min(psPrinterFeed->sPrinterFeed.ubNrOfPaperPaths, 3); iPath++) {
               CString strTmp; 
               strTmp.Format("[%4.4x] ", (WORD) psPrinterFeed->sPrinterFeed.wPaperPathStatus[iPath]);
               strItem.Append(strTmp);
            }
         }
         else strItem.Format("<UNDEFINED>");
      }
      else strItem = "<INVALID>";
      GetDlgItem(IDC_TXT_PFE_PAPER_PATH_STATUS)->SetWindowText(strItem);
   }

   //-- Update paper path  state
   { 
      //-- Regard paper path state valid if ...
      //..     printer feeder is valid 
      //.. AND age less than ... ms 
      if (psPrinterFeed && (dwAgeSec < 99999)) {
         if (psPrinterFeed->dwDataValidFlag & D_XPM_PMP_PFE_DVF_PRINTER_FEED) {
            strItem.Format("");
            for (int iPath = 0; iPath < min(psPrinterFeed->sPrinterFeed.ubNrOfPaperPaths, 3); iPath++) {
               CString strTmp;
               strTmp.Format("%d ", (WORD) psPrinterFeed->sPrinterFeed.ubPaperPathState[iPath]);
               strItem.Append(strTmp);
            }
         }
         else strItem.Format("<UNDEFINED>");
      }
      else strItem = "<INVALID>";
      GetDlgItem(IDC_TXT_PFE_PAPER_PATH_STATE)->SetWindowText(strItem);
   }
}

//******************************************************************************************
//** METHOD    CXpm_Pmpt_DlgMain::UpdateDialog
//** ABSTRACT  Retrieves registry data supplied by print monitor plugins and updates the 
//**           corresponding dialog items
//******************************************************************************************

void CXpm_Pmpt_DlgMain::UpdateDialog
   ()
{
   //-- Read job information from registry and update dialog 
   { 
      TS_XPM_PMP_JOB_INFO sJobInfo;  // Job information structure as stored in registry

      //-- Read job info from registry and check revision
      if (     ReadPrinterDataFromRegistry(m_pszPrinter, D_XPM_PMP_REG_KEY, D_XPM_PMP_REG_VALUE_PMP_JOB_INFO, (PBYTE) &sJobInfo, sizeof(sJobInfo))
           && (sJobInfo.ubRevision == D_XPM_PMP_JI_REVISION)
         ) UpdateJobInfoInDialog(&sJobInfo);
      else UpdateJobInfoInDialog(NULL);
   }

   //-- Read printer information from registry and update dialog
   { 
      TS_XPM_PMP_PRINTER_INFO sPrinterInfo; // Printer information structure as stored in registry

      //-- Read printer info from registry and check revision
      if (     ReadPrinterDataFromRegistry(m_pszPrinter, D_XPM_PMP_REG_KEY, D_XPM_PMP_REG_VALUE_PMP_PRINTER_INFO, (PBYTE) &sPrinterInfo, sizeof(sPrinterInfo))
           && (sPrinterInfo.ubRevision == D_XPM_PMP_PIN_REVISION)
         ) UpdatePrinterInfoInDialog(&sPrinterInfo);
      else UpdatePrinterInfoInDialog(NULL);
   }

   //-- Read status information from registry and update dialog
   { 
      TS_XPM_PMP_PRINTER_STATUS sPrinterStatus; // Printer status information structure as stored in registry

      //-- Read printer status from registry and check revision
      if (     ReadPrinterDataFromRegistry(m_pszPrinter, D_XPM_PMP_REG_KEY, D_XPM_PMP_REG_VALUE_PMP_PRINTER_STATUS, (PBYTE) &sPrinterStatus, sizeof(sPrinterStatus))
           && (sPrinterStatus.ubRevision == D_XPM_PMP_PST_REVISION)
         ) UpdatePrinterStatusInDialog(&sPrinterStatus);
      else UpdatePrinterStatusInDialog(NULL);
   }

   //-- Read feed unit information from registry and update dialog
   { 
      TS_XPM_PMP_PRINTER_FEED sPrinterFeed; // Feed unit status information structure as stored in registry

      //-- Read feed unit inforamtion from registry and check revision
      if (     ReadPrinterDataFromRegistry(m_pszPrinter, D_XPM_PMP_REG_KEY, D_XPM_PMP_REG_VALUE_PMP_PRINTER_FEED, (PBYTE) &sPrinterFeed, sizeof(sPrinterFeed))
           && (sPrinterFeed.ubRevision == D_XPM_PMP_PFE_REVISION)
         ) UpdatePrinterFeedInDialog(&sPrinterFeed);
      else UpdatePrinterFeedInDialog(NULL);
   }

   //-- Read presenter unit information from registry and update dialog
   { 
      TS_XPM_PMP_PRINTER_PRESENT sPrinterPresent; // Presenter unit status information structure as stored in registry

      //-- Read presenter unit information from registry and check revision
      if (     ReadPrinterDataFromRegistry(m_pszPrinter, D_XPM_PMP_REG_KEY, D_XPM_PMP_REG_VALUE_PMP_PRINTER_PRESENT, (PBYTE) &sPrinterPresent, sizeof(sPrinterPresent))
           && (sPrinterPresent.ubRevision == D_XPM_PMP_PPR_REVISION)
         ) UpdatePrinterPresentInDialog(&sPrinterPresent);
      else UpdatePrinterPresentInDialog(NULL);
   }

}

